@@ -5,7 +5,7 @@ from sentence_transformers import SentenceTransformer
 from sklearn.cluster import KMeans
 from sklearn.decomposition import PCA, TruncatedSVD
 from sklearn.preprocessing import MinMaxScaler
-from torchtext.data import BucketIterator, Field, TabularDataset
+from torch.utils.data import DataLoader
 from typing import List
 
 
@@ -56,40 +56,21 @@ class AutoEncoder(torch.nn.Module):
         :param decoder_hidden_layer: dict
             Input and output dimensions for each decoder hidden layer
         """
-        super().__init__()
-        self.encoder_hidden_layers: List[torch.nn] = []
-        self.decoder_hidden_layers: List[torch.nn] = []
-        for e in encoder_hidden_layer.keys():
-            self.encoder_hidden_layers.append(torch.nn.Linear(in_features=encoder_hidden_layer[e]['in_features'],
-                                                              out_features=encoder_hidden_layer[e]['out_features'],
-                                                              bias=True
-                                                              )
-                                              )
-        for d in decoder_hidden_layer.keys():
-            self.decoder_hidden_layers.append(torch.nn.Linear(in_features=decoder_hidden_layer[d]['in_features'],
-                                                              out_features=decoder_hidden_layer[d]['out_features'],
-                                                              bias=True
-                                                              )
-                                              )
+        super(AutoEncoder, self).__init__()
+        self.encoder_model: Encoder = Encoder(encoder_hidden_layer=encoder_hidden_layer)
+        self.decoder_model: Decoder = Decoder(decoder_hidden_layer=decoder_hidden_layer)
 
-    def forward(self, features) -> tuple:
+    def forward(self, features) -> torch.tensor:
         """
         Feed-Forward network
 
         :param features:
             Data input (features)
 
-        :return: tuple
-            Encoder weights and decoder weights
+        :return: torch.tensor
+            Encoder-decoder weights
         """
-        _encoder: torch.tensor = None
-        _decoder: torch.tensor = None
-        for encoder_layer in self.encoder_hidden_layers:
-            _encoder = torch.nn.LeakyReLU(encoder_layer(features))
-        _decoder = _encoder
-        for decoder_layer in self.decoder_hidden_layers:
-            _decoder = torch.nn.LeakyReLU(decoder_layer(features))
-        return _encoder, _decoder
+        return self.decoder_model(self.encoder_model(features))
 
 
 class Clustering(torch.nn.Module):
@@ -116,26 +97,96 @@ class Clustering(torch.nn.Module):
         self.weights_layer: torch.nn.Parameter = torch.nn.Parameter(data=torch.Tensor(in_features, out_features), requires_grad=False)
         self.weights_layer = torch.nn.init.xavier_uniform_(tensor=self.weights_layer, gain=1.0)
 
-    def forward(self, inputs: torch.tensor) -> torch.tensor:
+    def forward(self, x: torch.tensor) -> torch.tensor:
         """
         Feed-forward network
 
-        :param inputs:
-        :return:
+        :param x: torch.tensor
+            Encoder weights
+
+        :return: torch.tensor
+            Soft cluster assignment (measure similarity between embedding points and cluster centroids)
         """
-        _q = 1.0 / (1.0 + (torch.sum(torch.square(inputs.size()[1]) - self.n_clusters)) / self.alpha)
-        _q **= (self.alpha + 1.0) / 2.0
-        _q = (_q.transpose() / torch.sum(_q)).transpose()
+        self.weights_layer = torch.nn.Parameter(data=x, requires_grad=False)
+        _q = 1.0 / (1.0 + (torch.sum(torch.square(torch.unsqueeze(input=self.weights_layer, dim=2) - self.n_clusters)) / self.alpha))
+        _q = torch.pow(input=_q, exponent=(self.alpha + 1.0) / 2.0)
+        _q = (_q.transpose(dim0=0, dim1=0) / torch.sum(_q)).transpose(0, 0)
         return _q
 
-    def set_weight(self, weights: torch.tensor):
+    def set_weights(self, cluster_centers: np.ndarray):
         """
         Update weights
 
-        :param weights: torch.tensor
-            Weights
+        :param cluster_centers: np.ndarray
+            Clustering centers created by KMeans algorithm
         """
-        self.weights_layer = torch.nn.Parameter(data=weights, requires_grad=False)
+        self.weights_layer = torch.nn.Parameter(data=torch.tensor(data=cluster_centers), requires_grad=False)
+
+
+class Decoder(torch.nn.Module):
+    """
+    Class for building decoder network using PyTorch
+    """
+    def __init__(self, decoder_hidden_layer: dict):
+        """
+        :param decoder_hidden_layer: dict
+            Input and output dimensions for each decoder hidden layer
+        """
+        super(Decoder, self).__init__()
+        self.decoder_model: torch.nn.Sequential = torch.nn.Sequential()
+        for d in decoder_hidden_layer.keys():
+            self.decoder_model.add_module(name='decoder_{}'.format(d),
+                                          module=torch.nn.Linear(in_features=decoder_hidden_layer[d]['in_features'],
+                                                                 out_features=decoder_hidden_layer[d]['out_features'],
+                                                                 bias=True
+                                                                 )
+                                          )
+            self.decoder_model.add_module(name='decoder_{}_activation'.format(d), module=torch.nn.LeakyReLU())
+
+    def forward(self, features) -> torch.tensor:
+        """
+        Feed-Forward network
+
+        :param features:
+            Data input (features)
+
+        :return: torch.tensor
+            Decoder weights
+        """
+        return self.decoder_model(features)
+
+
+class Encoder(torch.nn.Module):
+    """
+    Class for building encoder network using PyTorch
+    """
+    def __init__(self, encoder_hidden_layer: dict):
+        """
+        :param encoder_hidden_layer: dict
+            Input and output dimensions for each encoder hidden layer
+        """
+        super(Encoder, self).__init__()
+        self.encoder_model: torch.nn.Sequential = torch.nn.Sequential()
+        for e in encoder_hidden_layer.keys():
+            self.encoder_model.add_module(name='encoder_{}'.format(e),
+                                          module=torch.nn.Linear(in_features=encoder_hidden_layer[e]['in_features'],
+                                                                 out_features=encoder_hidden_layer[e]['out_features'],
+                                                                 bias=True
+                                                                 )
+                                          )
+            self.encoder_model.add_module(name='encoder_{}_activation'.format(e), module=torch.nn.LeakyReLU())
+
+    def forward(self, features) -> torch.tensor:
+        """
+        Feed-Forward network
+
+        :param features:
+            Data input (features)
+
+        :return: torch.tensor
+            Encoder weights
+        """
+        return self.encoder_model(features)
 
 
 class SIF:
@@ -227,10 +278,10 @@ class STC(torch.nn.Module):
                  encoder_hidden_layer: dict,
                  decoder_hidden_layer: dict,
                  dimensions: List[int],
-                 iterator: BucketIterator,
+                 iterator: DataLoader,
                  batch_size: int,
                  epoch: int = 15,
-                 optimizer: str = 'kld',
+                 optimizer: str = 'rmsprop',
                  n_clusters: int = 20,
                  alpha: float = 1.0,
                  n_iterations: int = 100,
@@ -294,16 +345,20 @@ class STC(torch.nn.Module):
         self.epoch: int = epoch
         self.batch_size: int = batch_size
         self.optimizer: str = optimizer
-        self.iterator: BucketIterator = iterator
-        self.encoder, self.auto_encoder = AutoEncoder(encoder_hidden_layer=encoder_hidden_layer,
-                                                      decoder_hidden_layer=decoder_hidden_layer
-                                                      )
+        self.iterator: DataLoader = iterator
+        self.model: torch.nn.Sequential = torch.nn.Sequential()
+        self.encoder = Encoder(encoder_hidden_layer=encoder_hidden_layer)
+        self.decoder = Decoder(decoder_hidden_layer=decoder_hidden_layer)
+        self.auto_encoder = AutoEncoder(encoder_hidden_layer=encoder_hidden_layer, decoder_hidden_layer=decoder_hidden_layer)
+        print('Pre-train AutoEncoder ...')
+        self.pre_train_auto_encoder()
         _clustering_layer = Clustering(n_clusters=self.n_clusters,
-                                       in_features=10,
-                                       out_features=10,
+                                       in_features=20,
+                                       out_features=20,
                                        alpha=self.alpha
-                                       )(self.encoder)
-        self.model: torch.nn.Sequential = torch.nn.Sequential(self.encoder, _clustering_layer)
+                                       )
+        self.model.add_module(name='encoder', module=self.encoder)
+        self.model.add_module(name='cluster', module=_clustering_layer)
 
     def _clip_gradient(self, clip_value: float):
         """
@@ -326,14 +381,17 @@ class STC(torch.nn.Module):
         :return: torch.tensor
             Embedding feature tensor
         """
-        return self.encoder.predict(x=x)
+        return self.encoder(x)
 
-    def fit(self) -> tuple:
+    def fit(self, x: torch.tensor) -> tuple:
         """
-        Train STC using PyTorch
+        Train STC using PyTorch and KMeans from sklearn
+
+        :param x: torch.tensor
+            Embedding values
 
         :return: tuple
-            Observed label and label prediction
+            Observed label and predicted label
         """
         _predictions: List[int] = []
         _observations: List[int] = []
@@ -341,67 +399,49 @@ class STC(torch.nn.Module):
             self.model.cuda()
         self.model.train()
         if self.optimizer == 'rmsprop':
-            _optim: torch.optim = torch.optim.RMSprop(
-                params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                lr=0.0001
-                )
+            _optim: torch.optim = torch.optim.RMSprop(params=self.model.parameters(), lr=0.0001)
         elif self.optimizer == 'adam':
-            _optim: torch.optim = torch.optim.Adam(params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                                                   lr=0.0001
-                                                   )
+            _optim: torch.optim = torch.optim.Adam(params=self.model.parameters(), lr=0.0001)
         elif self.optimizer == 'sgd':
-            _optim: torch.optim = torch.optim.SGD(params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                                                  lr=0.0001
-                                                  )
+            _optim: torch.optim = torch.optim.SGD(params=self.model.parameters(), lr=0.0001)
         else:
             raise SelfTaughtShortTextClusteringException('Optimizer ({}) not supported'.format(self.optimizer))
-        for e in range(0, self.epoch, 1):
-            for idx, batch in enumerate(self.iterator):
-                _predictors = batch.text[0]
-                _target = batch.label
-                if _target.size()[0] != self.batch_size:
-                    continue
-                if torch.cuda.is_available():
-                    _target = _target.cuda()
-                    _predictors = _predictors.cuda()
-                _optim.zero_grad()
-                _prediction = torch.softmax(input=self.model(_predictors), dim=1)
-                _top_class, _top_p = _prediction.topk(k=1, dim=1)
-                _predictions.extend(_top_class.detach().tolist())
-                _observations.extend(_target.detach().numpy().tolist())
-                _loss = torch.nn.KLDivLoss(_prediction, _target)
-                _loss.backward()
-                self._clip_gradient(clip_value=1e-1)
-                _optim.step()
-        return _observations, _predictions
-
-    def forward(self, x: np.ndarray) -> np.ndarray:
-        """
-        Feed-forward network
-
-        :param x: np.ndarray
-            Embedding values
-
-        :return: np.ndarray
-            Label prediction
-        """
-        _nmi: int = 0
+        _loss_function: torch.nn.KLDivLoss = torch.nn.KLDivLoss()
         _k_means: KMeans = KMeans(n_clusters=self.n_clusters, n_init=self.n_iterations)
-        _pred: np.ndarray = _k_means.fit_predict(self.encoder.predict(x))
+        _pred: np.ndarray = _k_means.fit_predict(self.encoder(x).detach().numpy())
         _last_pred: np.ndarray = np.copy(_pred)
-        self.model.set_weights([_k_means.cluster_centers_])
+        self.model[1].set_weights(cluster_centers=_k_means.cluster_centers_)
+        _index: int = 0
+        _index_array: np.ndarray = np.arange(x.shape[0])
         for i in range(0, self.max_iterations, 1):
+            print('Iteration: ', i)
             if i % self.update_interval == 0:
-                _q = self.model.predict(x)
+                _q = self.model(x)
                 _p = self.target_distribution(_q)
-                _pred = _q.argmax(1)
-                #_nmi = np.round(normalized_mutual_info_score(labels_true=y, labels_pred=_pred, average_method='arithmetic'))
+                _pred = _q.argmax(-1)
                 # check stopping criterion
+                print('_pred', _pred)
+                print('_pred.shape', _pred.shape)
+                print('_last_pred', _last_pred)
                 _delta_label = np.sum(_pred != _last_pred).astype(np.float32) / _pred.shape[0]
                 _last_pred = np.copy(_pred)
                 if i > 0 and _delta_label < self.tol:
                     break
-        return _pred
+            _idx: np.ndarray = _index_array[_index * self.batch_size: min((_index + 1) * self.batch_size, x.shape[0])]
+            _predictors, _target = self.iterator.dataset[_idx]
+            if torch.cuda.is_available():
+                _predictors = _predictors.cuda()
+            _optim.zero_grad()
+            _prediction = torch.softmax(input=self.model(_predictors), dim=0)
+            _top_class, _top_p = _prediction.topk(k=1, dim=0)
+            _predictions.extend(_top_class.detach().tolist())
+            _observations.extend(_target.detach().numpy().tolist())
+            _loss = _loss_function(_prediction, _p[_idx])
+            _loss.backward()
+            self._clip_gradient(clip_value=1e-1)
+            _optim.step()
+            _index = _index + 1 if (_index + 1) * self.batch_size <= x.shape[0] else 0
+        return _observations, _predictions
 
     def predict(self, x: np.ndarray) -> int:
         """
@@ -413,60 +453,36 @@ class STC(torch.nn.Module):
         :return: int
             Label value
         """
-        _q: np.ndarray = self.model.predict(x)
-        return _q.argmax(1)
+        _q: np.ndarray = self.model(x)
+        return _q.argmax(0)
 
-    def pre_train_auto_encoder(self,
-                               iterator: BucketIterator,
-                               epoch: int = 15,
-                               optimizer: str = 'rmsprop'
-                               ):
+    def pre_train_auto_encoder(self):
         """
         Pre-train auto-encoder network
-
-        :param iterator: BucketIterator
-            Batched data set
-
-        :param epoch: int
-            Number of epochs
-
-        :param optimizer: str
-            Name of the optimizer:
-                -> rmsprop: RMSprop
-                -> adam: Adam
-                -> sgd: Stochastic Gradient Decent
         """
-        _predictions: List[int] = []
-        _observations: List[int] = []
         if torch.cuda.is_available():
             self.auto_encoder.cuda()
         self.auto_encoder.train()
-        if optimizer == 'rmsprop':
-            _optim: torch.optim = torch.optim.RMSprop(params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                                                      lr=0.0001
-                                                      )
-        elif optimizer == 'adam':
-            _optim: torch.optim = torch.optim.Adam(params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                                                   lr=0.0001
-                                                   )
-        elif optimizer == 'sgd':
-            _optim: torch.optim = torch.optim.SGD(params=filter(lambda p: p.grad is not None, self.model.parameters()),
-                                                  lr=0.0001
-                                                  )
+        if self.optimizer == 'rmsprop':
+            _optim: torch.optim = torch.optim.RMSprop(params=self.auto_encoder.parameters(), lr=0.0001)
+        elif self.optimizer == 'adam':
+            _optim: torch.optim = torch.optim.Adam(params=self.auto_encoder.parameters(), lr=0.0001)
+        elif self.optimizer == 'sgd':
+            _optim: torch.optim = torch.optim.SGD(params=self.auto_encoder.parameters(), lr=0.0001)
         else:
-            raise SelfTaughtShortTextClusteringException('Optimizer ({}) not supported'.format(optimizer))
-        for e in range(0, epoch, 1):
-            for idx, batch in enumerate(iterator):
-                _predictors = batch.text[0]
-                _target = batch.label
-                if _target.size()[0] != self.batch_size:
+            raise SelfTaughtShortTextClusteringException('Optimizer ({}) not supported'.format(self.optimizer))
+        _loss_function: torch.nn.MSELoss = torch.nn.MSELoss()
+        for e in range(0, self.epoch, 1):
+            print('Epoch: ', e)
+            for idx, batch in enumerate(self.iterator):
+                _predictors = torch.autograd.Variable(batch[0])
+                if _predictors.size()[0] != self.batch_size:
                     continue
                 if torch.cuda.is_available():
-                    _target = _target.cuda()
                     _predictors = _predictors.cuda()
                 _optim.zero_grad()
-                _prediction = torch.softmax(input=self.auto_encoder(_predictors), dim=1)
-                _loss = torch.nn.MSELoss(_prediction, _target)
+                _prediction = self.auto_encoder(_predictors)
+                _loss = _loss_function(_prediction, _predictors)
                 _loss.backward()
                 self._clip_gradient(clip_value=1e-1)
                 _optim.step()
@@ -476,8 +492,9 @@ class STC(torch.nn.Module):
         """
         Auxiliary target distribution
 
-        :param q:
+        :param q: torch.tensor
+
         :return:
         """
         _weight = q ** 2 / q.sum(0)
-        return (_weight.transpose / _weight.sum(1)).transpose
+        return (_weight.transpose(0, 0) / _weight.sum(0)).transpose(0, 0)
