@@ -46,7 +46,7 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
                                                  convergence=True,
-                                                 convergence_measure='min',
+                                                 convergence_measure='median',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
                                                  plot=False,
@@ -93,7 +93,7 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
                                                  convergence=True,
-                                                 convergence_measure='min',
+                                                 convergence_measure='median',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
                                                  plot=False,
@@ -144,7 +144,7 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
                                                  convergence=True,
-                                                 convergence_measure='min',
+                                                 convergence_measure='median',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
                                                  plot=False,
@@ -184,13 +184,13 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  warm_start_strategy='monotone',
                                                  warm_start_constant_hidden_layers=0,
                                                  warm_start_constant_category='very_small',
-                                                 max_generations=5,
-                                                 pop_size=4,
+                                                 max_generations=1,
+                                                 pop_size=64,
                                                  mutation_rate=0.1,
                                                  mutation_prob=0.15,
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
-                                                 convergence=True,
+                                                 convergence=False,
                                                  convergence_measure='min',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
@@ -203,19 +203,19 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  sampling_function=None
                                                  )
         _ga.optimize()
-        _ga.visualize(results_table=True,
+        _ga.visualize(results_table=False,
                       model_distribution=True,
                       model_evolution=True,
-                      param_distribution=True,
+                      param_distribution=False,
                       train_time_distribution=True,
                       breeding_map=True,
-                      breeding_graph=True,
+                      breeding_graph=False,
                       fitness_distribution=True,
                       fitness_evolution=True,
                       fitness_dimensions=True,
                       per_generation=True,
-                      prediction_of_best_model=True,
-                      epoch_stats=True
+                      prediction_of_best_model=False,
+                      epoch_stats=False
                       )
         _found_plot: List[bool] = []
         for plot in ['ga_metadata_table',
@@ -271,7 +271,7 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
                                                  convergence=True,
-                                                 convergence_measure='min',
+                                                 convergence_measure='median',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
                                                  plot=False,
@@ -350,7 +350,7 @@ class GeneticAlgorithmTest(unittest.TestCase):
                                                  parents_ratio=0.5,
                                                  early_stopping=0,
                                                  convergence=True,
-                                                 convergence_measure='min',
+                                                 convergence_measure='median',
                                                  timer_in_seconds=43200,
                                                  force_target_type=None,
                                                  plot=False,
